//------------------------------------------------------------
/// \file	Main.cpp
/// \author	Rob Bateman
/// \date	9-feb-2005
/// \brief	This shows how to make your code frame rate
///			independant. If you change the window size the
///			object will still spin at the same speed.
//------------------------------------------------------------

#include <stdlib.h>
#include <GL/glut.h>

/// a structure to hold a mouse click. It also hold a pointer
/// to the next element in the stack.
struct Click {

	/// the screen position
	int position[2];

	/// constructor
	Click(int x,int y) {

		// set position
		position[0] = x;
		position[1] = y;

		next = 0;
	}

	/// pointer to the next click
	Click* next;
};

/// the top of the stack
Click* top=0;

//------------------------------------------------------------	Push()
//
/// pushes a new data element onto the stack
void Push(int x,int y) {

	// create a new click structure to hold our data
	Click* p = new Click(x,y);

	// set the current top of the stack to be the next
	// of our new element.
	p->next = top;

	// set the top to the new element
	top = p;
}

//------------------------------------------------------------	Pop()
//
/// removes the top element from the stack
void Pop() {

	// ignore an empty stack
	if(!top)
		return;

	// store temp reference to the element on the top
	Click* p = top;

	// get the top to reference the item underneath it
	top = top->next;

	// delete the item
	delete p;
}

//------------------------------------------------------------	DrawParticles()
//
void DrawClicks() {

	// iterate over all stack elements and draw a point
	Click* curr = top;

	// draw lines between stack elements
	glColor3f(1,0,0);
	glBegin(GL_LINE_STRIP);

	// while not at end of stack
	while (curr) {

		// draw point
		glVertex2iv(curr->position);

		// move to next element
		curr = curr->next;
	}
	glEnd();


	// draw points at each stored item
	curr = top;
	glColor3f(1,1,0);
	glPointSize(2);
	glBegin(GL_POINTS);
	// while not at end of stack
	while (curr) {

		// draw point
		glVertex2iv(curr->position);

		// move to next element
		curr = curr->next;
	}
	glEnd();
}


//------------------------------------------------------------	OnReshape()
//
void OnReshape(int w, int h)
{
	// prevent division by zero when minising window
	if (h==0)
		h=1;

	// set the drawable region of the window
	glViewport(0,0,w,h);

	// set up the projection matrix
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	// just use a perspective projection
	glOrtho(0,w,h,0,-100,100);

	// go back to modelview matrix so we can move the objects about
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

//------------------------------------------------------------	Draw()
//
void OnDraw() {

	// clear the screen & depth buffer
	glClear(GL_DEPTH_BUFFER_BIT|GL_COLOR_BUFFER_BIT);

	// clear the previous transform
	glLoadIdentity();

	// draw the mouse clicks
	DrawClicks();

	// currently we've been drawing to the back buffer, we need
	// to swap the back buffer with the front one to make the image visible
	glutSwapBuffers();
}

//------------------------------------------------------------	OnInit()
//
void OnInit() {
	// enable depth testing
	glEnable(GL_DEPTH_TEST);
}

//------------------------------------------------------------	OnExit()
//
void OnExit() {
}

//------------------------------------------------------------	OnExit()
//
void OnMouseClick(int button,int state,int x,int y) {

	// if button pressed
	if(state == GLUT_DOWN) {

		// push the x and y coord of the mouse click onto the stack
		if (button == GLUT_LEFT_BUTTON)
			Push(x,y);

		// remove the top element from the stack
		if (button == GLUT_RIGHT_BUTTON)
			Pop();
	}
	glutPostRedisplay();
}

//------------------------------------------------------------	main()
//
int main(int argc,char** argv) {

	// initialise glut
	glutInit(&argc,argv);

	// request a depth buffer, RGBA display mode, and we want double buffering
	glutInitDisplayMode(GLUT_DEPTH|GLUT_RGBA|GLUT_DOUBLE);

	// set the initial window size
	glutInitWindowSize(640,480);

	// create the window
	glutCreateWindow("A basic stack example");

	// set the function to use to draw our scene
	glutDisplayFunc(OnDraw);

	// set the function to handle changes in screen size
	glutReshapeFunc(OnReshape);

	// set the idle callback
	glutMouseFunc(OnMouseClick);

	// run our custom initialisation
	OnInit();

	// set the function to be called when we exit
	atexit(OnExit);

	// this function runs a while loop to keep the program running.
	glutMainLoop();
	return 0;
}
